@@ -9,6 +9,10 @@ st_as_stars.Raster = function(.x, ..., att = 1, ignore_file = FALSE) {
 	#0 360 -90  90
 	e = as.vector(raster::extent(.x)) # xmin xmax ymin ymax
 
+	RasterIO = if ("data" %in% slotNames(.x) && inherits(.x@data, ".SingleLayerData"))
+			list(bands = .x@data@band)
+		else
+			list()
 	if (!ignore_file) {
 		file = if ("file" %in% slotNames(.x))
 				.x@file@name
@@ -17,38 +21,13 @@ st_as_stars.Raster = function(.x, ..., att = 1, ignore_file = FALSE) {
 			else
 				""
 		if (file != "") {
-			r = try(read_stars(file, ...), silent = TRUE)
+			r = try(read_stars(file, RasterIO = RasterIO, ...), silent = TRUE)
 			if (!inherits(r, "try-error")) {
 
 				if (is.na(st_crs(r)))
 					r = st_set_crs(r, st_crs(raster::crs(.x)))
 
-				rdims = attr(r, "dimensions")
-				dxy = attr(rdims, "raster")$dimensions
-				dimx = rdims[[dxy[1]]]
-				dimy = rdims[[dxy[2]]]
-
-				xrev = dimx$delta < 0
-				yrev = dimy$delta < 0
-
-				if (xrev) {
-					dimx$offset = e[2]
-					dimx$delta = (e[1] - e[2]) / (dimx$to - dimx$from + 1)
-				} else {
-					dimx$offset = e[1]
-					dimx$delta = (e[2] - e[1]) / (dimx$to - dimx$from + 1)
-				}
-
-				if (yrev) {
-					dimy$offset = e[4]
-					dimy$delta = (e[3] - e[4]) / (dimy$to - dimy$from + 1)
-				} else {
-					dimy$offset = e[3]
-					dimy$delta = (e[4] - e[3]) / (dimy$to - dimy$from + 1)
-				}
-
-				attr(r, "dimensions")[[dxy[1]]] = dimx
-				attr(r, "dimensions")[[dxy[2]]] = dimy
+				r = fix_dims(r, e)
 
 				if (inherits(.x, "RasterLayer"))
 					names(r) <- names(.x)
@@ -95,54 +74,6 @@ st_as_stars.Raster = function(.x, ..., att = 1, ignore_file = FALSE) {
 	else
 		ret
 }
-
-st_as_raster = function(x, ...) {
-	stopifnot(inherits(x, "stars"))
-	x = st_upfront(x) # x/y dimensions first
-	if (length(dim(x)) > 3) {
-		warning("folding all higher dimensions into the third dimension") # nocov
-		x = st_apply(x, 1:2, as.vector) # fortunes::fortune("side effect") # nocov
-	}
-	if (length(dim(x)) == 2 && length(x) > 1)
-		x = merge(x)
-	d = st_dimensions(x)
-	if (d[[2]]$delta > 0) { # swap:
-		ny = dim(x)[2]
-		d[[2]]$offset = d[[2]]$offset + ny * d[[2]]$delta # top
-		d[[2]]$delta = -d[[2]]$delta # going down
-		x[[1]] = if (length(dim(x)) == 2)
-				x[[1]][,ny:1]
-			else
-				x[[1]][,ny:1,]
-	}
-	dxy = attr(d, "raster")$dimensions
-	stopifnot(all(dxy %in% names(d)))
-	bb = st_bbox(x)
-	values = if (is.factor(x[[1]]))
-			structure(x[[1]], dim = NULL)
-		else
-			as.vector(x[[1]]) # would convert factor into character
-	if (length(dim(x)) == 2) {
-		raster::raster(nrows=dim(x)[ dxy[2] ], ncols=dim(x)[ dxy[1] ],
-			xmn = bb[1], xmx = bb[3], ymn = bb[2], ymx = bb[4],
-			crs = as(st_crs(x), "CRS"), vals = values)
-	} else {
-		third = setdiff(names(d), dxy)
-		b = raster::brick(nrows=dim(x)[ dxy[2] ], ncols=dim(x)[ dxy[1] ],
-			xmn = bb[1], xmx = bb[3], ymn = bb[2], ymx = bb[4], nl = dim(x)[third],
-			crs = as(st_crs(x), "CRS"))
-		raster::values(b) = values
-		z = seq(d[[third]])
-		if (!any(is.na(z))) {
-			if (is.character(z))
-				names(b) = z
-			else
-				b = raster::setZ(b, z)
-		}
-		b
-	}
-}
-
 #' Coerce stars object into a Raster raster or brick
 #'
 #' Coerce stars object into a Raster raster or brick
@@ -160,7 +91,7 @@ setAs("stars", "Raster", function(from) {
 		stop("package raster required, please install it first") # nocov
 	if (!is_regular_grid(from))
 		stop("only regular rasters can be converted to Raster* objects")
-	st_as_raster(from)
+	st_as_raster(from, class = "Raster")
 })
 
 setAs("stars_proxy", "Raster", function(from) {
@@ -176,42 +107,189 @@ setAs("stars_proxy", "Raster", function(from) {
 	raster::brick(unlist(from))
 })
 
-
+#' @name st_as_stars
+#' @param ignore_file logical; if \code{TRUE}, ignore the SpatRaster object file name
 #' @export
-st_bbox.SpatRaster = function(obj, ...) {
+st_as_stars.SpatRaster = function(.x, ..., ignore_file = FALSE) {
 	if (!requireNamespace("terra", quietly = TRUE))
 		stop("package terra required, please install it first") # nocov
-	bb = as.vector(terra::ext(obj))[c(1,3,2,4)]
-	names(bb) = c("xmin", "ymin", "xmax", "ymax")
-	st_bbox(bb, crs = st_crs(obj))
+
+	#0 360 -90  90
+	e = as.vector(terra::ext(.x)) # xmin xmax ymin ymax
+
+	RasterIO = list()
+	if (!ignore_file) {
+		file = terra::sources(.x)$source
+		if (file != "") {
+			r = try(read_stars(file, RasterIO = RasterIO, ...), silent = TRUE)
+			if (!inherits(r, "try-error")) {
+				if (is.na(st_crs(r)))
+					r = st_set_crs(r, st_crs(terra::crs(.x)))
+				r = fix_dims(r, e)
+				return(r)
+			}
+		}
+	}
+
+	v = terra::values(.x, mat = FALSE)
+	dim(v) = dim(.x)[c(2,1,3)]
+	if (all(terra::is.factor(.x))) {
+		l = terra::levels(.x)[[1]]
+		colors = try(rgb(terra::coltab(.x)[[1]], maxColorValue = 255))
+		if (inherits(colors, "try-error") || length(colors) == 0)
+			colors = NULL
+		else
+			colors = colors[-length(colors)]
+		v = structure(v + 1, class = "factor", levels = l, colors = colors)
+		# FIXME: should we handle levels for all layers here, or break on multiple different ones?
+	}
+	dimensions = list(
+		x = create_dimension(from = 1, to = dim(v)[1], offset = e[1],
+							 delta = (e[2]-e[1])/dim(v)[1], refsys = st_crs(terra::crs(.x))),
+		y = create_dimension(from = 1, to = dim(v)[2], offset = e[4],
+							 delta = (e[3]-e[4])/dim(v)[2], refsys = st_crs(terra::crs(.x))))
+	dimensions$band = create_dimension(values = names(.x))
+	l = if (length(names) > 1)
+		setNames(list(v), deparse(substitute(.x), 50))
+	else
+		setNames(list(v), names(.x)[1])
+	ret = st_as_stars(l, dimensions = create_dimensions(dimensions, get_raster()))
+	if (dim(ret)[3] == 1)
+		adrop(ret, 3)
+	else
+		ret
 }
 
-#' @export
-st_crs.SpatRaster = function(x, ...) {
+#' Coerce stars object into a terra SpatRaster
+#'
+#' Coerce stars object into a terra SpatRaster
+#' @param from object to coerce
+#' @name as
+#' @rdname coerce-methods
+#' @aliases coerce,stars,Terra-method
+#' @aliases coerce,stars_proxy,Terra-method
+#' @returns SpatRaster
+#' @details If the stars object has more than three dimensions, all dimensions higher than the third will be collapsed into the third dimensions. If the stars object has only an x/y raster but multiple attributes, these are merged first, then put in a SpatRaster.
+setAs("stars", "SpatRaster", function(from) {
 	if (!requireNamespace("terra", quietly = TRUE))
 		stop("package terra required, please install it first") # nocov
-	st_crs(terra::crs(x))
+	if (!is_regular_grid(from))
+		stop("only regular rasters can be converted to SpatRaster objects")
+	st_as_raster(from, class = "SpatRaster")
+})
+
+setAs("stars_proxy", "SpatRaster", function(from) {
+	if (!requireNamespace("terra", quietly = TRUE))
+		stop("package terra required, please install it first") # nocov
+	if (!is_regular_grid(from))
+		stop("only regular rasters can be converted to SpatRaster objects")
+	if (length(attr(from, "call_list"))) {
+		fname = paste0(tempfile(), ".tif")
+		write_stars(from, fname)
+		from = fname
+	}
+	terra::rast(unlist(from))
+})
+
+fix_dims = function(r, e){
+	e = unname(e)
+	rdims = attr(r, "dimensions")
+	dxy = attr(rdims, "raster")$dimensions
+	dimx = rdims[[dxy[1]]]
+	dimy = rdims[[dxy[2]]]
+
+	xrev = dimx$delta < 0
+	yrev = dimy$delta < 0
+
+	if (xrev) {
+		dimx$offset = e[2]
+		dimx$delta = (e[1] - e[2]) / (dimx$to - dimx$from + 1)
+	} else {
+		dimx$offset = e[1]
+		dimx$delta = (e[2] - e[1]) / (dimx$to - dimx$from + 1)
+	}
+
+	if (yrev) {
+		dimy$offset = e[4]
+		dimy$delta = (e[3] - e[4]) / (dimy$to - dimy$from + 1)
+	} else {
+		dimy$offset = e[3]
+		dimy$delta = (e[4] - e[3]) / (dimy$to - dimy$from + 1)
+	}
+
+	attr(r, "dimensions")[[dxy[1]]] = dimx
+	attr(r, "dimensions")[[dxy[2]]] = dimy
+
+	r
 }
 
-#' @export
-st_as_stars.SpatRaster = function(.x, ...) {
-	if (!requireNamespace("terra", quietly = TRUE))
-		stop("package terra required, please install it first") # nocov
-	if (!requireNamespace("raster", quietly = TRUE))
-		stop("package terra required, please install it first") # nocov
-	st_as_stars(as(.x, "Raster"), ...)
-}
-
-st_bbox.SpatVector = function(obj, ...) {
-	if (!requireNamespace("terra", quietly = TRUE))
-		stop("package terra required, please install it first") # nocov
-    bb = as.vector(terra::ext(obj))[c(1,3,2,4)]
-    names(bb) = c("xmin", "ymin", "xmax", "ymax")
-    st_bbox(bb, crs = st_crs(obj))
-}
-
-st_crs.SpatVector = function(x, ...) {
-	if (!requireNamespace("terra", quietly = TRUE))
-		stop("package terra required, please install it first") # nocov
-    st_crs(terra::crs(x))
+st_as_raster = function(x, class, ...) {
+	stopifnot(inherits(x, "stars"))
+	x_crs = st_crs(x)
+	x = st_upfront(x) # x/y dimensions first
+	if (length(dim(x)) > 3) {
+		warning("folding all higher dimensions into the third dimension") # nocov
+		x = st_apply(x, 1:2, as.vector) # fortunes::fortune("side effect") # nocov
+	}
+	if (length(dim(x)) == 2 && length(x) > 1)
+		x = merge(x)
+	d = st_dimensions(x)
+	if (d[[2]]$delta > 0) { # swap:
+		ny = dim(x)[2]
+		d[[2]]$offset = d[[2]]$offset + ny * d[[2]]$delta # top
+		d[[2]]$delta = -d[[2]]$delta # going down
+		x[[1]] = if (length(dim(x)) == 2)
+			x[[1]][,ny:1]
+		else
+			x[[1]][,ny:1,]
+	}
+	dxy = attr(d, "raster")$dimensions
+	stopifnot(all(dxy %in% names(d)))
+	bb = st_bbox(x)
+	values = if (is.factor(x[[1]])) {
+		structure(x[[1]], dim = NULL)
+	} else {
+		as.vector(x[[1]]) # would convert factor into character
+	}
+	if (class == "SpatRaster"){
+		third = setdiff(names(d), dxy)
+		b = terra::rast(nrows = dim(x)[ dxy[2] ], ncols=dim(x)[ dxy[1] ],
+						xmin = bb[1], xmax = bb[3], ymin = bb[2], ymax = bb[4],
+						nlyrs = ifelse(length(dim(x)) == 2, 1, dim(x)[third]),
+						crs = x_crs$wkt)
+		terra::values(b) = values
+		if (all(vapply(x, is.factor, FUN.VALUE = logical(1)))) {
+			terra::coltab(b) = lapply(x, function(x) t(col2rgb(attr(x, "colors"), alpha = TRUE)))
+		}
+		if (length(dim(x)) != 2){
+			z = seq(d[[third]])
+			if (!any(is.na(z))) {
+				if (is.character(z)) {
+					names(b) = z
+				} else {
+					names(b) = paste0(third, z)
+				}
+			}
+		}
+	} else {
+		if (length(dim(x)) == 2) {
+			b = raster::raster(nrows=dim(x)[ dxy[2] ], ncols=dim(x)[ dxy[1] ],
+							   xmn = bb[1], xmx = bb[3], ymn = bb[2], ymx = bb[4],
+							   crs = as(x_crs, "CRS"), vals = values)
+		} else {
+			third = setdiff(names(d), dxy)
+			b = raster::brick(nrows=dim(x)[ dxy[2] ], ncols=dim(x)[ dxy[1] ],
+							  xmn = bb[1], xmx = bb[3], ymn = bb[2], ymx = bb[4], nl = dim(x)[third],
+							  crs = as(x_crs, "CRS"))
+			raster::values(b) = values
+			z = seq(d[[third]])
+			if (!any(is.na(z))) {
+				if (is.character(z))
+					names(b) = z
+				else
+					b = raster::setZ(b, z)
+			}
+		}
+	}
+	return(b)
 }
