@@ -1,6 +1,6 @@
 ## load dependencies
 library(testthat)
-library(stars)
+suppressPackageStartupMessages(library(stars))
 
 ## test package
 test_check("stars")
